;
;	 **********************
;	 *                    *
;	 *  GRENADE LOBBER 2  *
;	 *                    *
;	 **********************
;
;   	    by DAVE HASSLER
;       for the PAL-1, microKIM,
; and KIM-1 single-board 6502 computers
;
;
;   This version of GRENADE LOBBER (Part Deux? Electric Bugaloo??)
; utilizes the PAL-1's LED display and onboard keypad for game play.  
; It will fit (barely) into a stock KIM-1 with only 1K RAM.  To do
; this, I used the bottom of the stack and RIOT RAM to get it all in.
; It's not the prettiest code in the world, aside from the stuff I
; copied from others, but it plays OK.  I wrote this version to learn
; about the RIOT's I/O registers and basic interfacing, bit
; manipulation math, and making custom display characters.
;
;   Game Play: the game 'field' is 50 yards long. Your base is in
; locations 1-4.  The 'adversary' wants to blow up your fort, and
; starts somewhere between 20 and 50 yards from you. You 'lob' a
; grenade at him by entering the distance to throw at the prompt
; 'dISt _ _'.  Throws over 50 yards will be capped at 50 yards.
; If the grenade lands within +/- 1 yard, the foe is eliminated. If
; not, you get a report for that shot. E.g., on a throw 42 yards out
; from your fort: 'LonG 42' and then how many grenades are left.
;    If missed, the adversary will advance on your position anywhere
; from 1-7 yards.  You only have three grenades.  You lose if you run
; out of grenades or the foe eludes your defense and captures you.

;   Zero Page locations used: $70-76, $80-84, $8C-91.
; Also, Page 1 from $0100-01B0, and RIOT RAM from $1780-17B0.

STOREA = $70
NMEPOS = $71
MISSBY = $72
GRNADE = $73
TARGET = $74
STOREX = $75
STOREY = $76

* = $0200

	CLD
	LDA #$FF
	STA $F2		; SET STACK POINTER
; INIT AND TITLE

SEEDST	LDA $1744	; from 1 u-sec timer in the RIOT
	BEQ SEEDST	; if #00, get another 'seed'
	STA R_SEED	; store it in main RND routine seed holder

TITLE	LDA #$00
	STA STOREA	; CLEAR AC TEMP LOC.
	STA $83		; LOOP COUNTER
	LDY #$1E	; LOAD MSG #6
	STY STOREY
	JSR MESAGE	

NME1ST	JSR RAND_8	; get a random number 00-FF in A
	AND #$3F	; mask to keep bottom 6 bits (<#64) 
	CMP #$14	; is A <#20?
	BCC NME1ST	; yes, get another rnd #
	CMP #$32	; is A >#50?
	BCS NME1ST	; yes, get another rnd #
	STA NMEPOS	; store the AC in 'Enemy Pos.' location
	LDA #$03	; HEX #
	STA GRNADE	; number of grenades



; *** MAIN LOOP ***

MAIN	JSR PLYRIN	; GET PLAYER INPUT
	JSR TESTIT	; IS THE INPUT GOOD?
	NOP
	JMP MAIN



; *** SUBROUTINES ***

LOBBIT	LDA #$24	; START FOR MSGS 7-12
	STA $83
LOADGR	TAY
	JSR LODMSG
	JSR DELAY1
	CLC
	LDA $83		; GET BACK THE MSG # AND +6
	ADC #$06	; (ALL MSGS ARE 6 CHAR.)
	STA $83
	CMP #$48	; DON'T WANT MSG #13
	BNE LOADGR
	RTS

; GET PLAYER INPUT

PLYRIN	LDY #$00	; MSG #1 - 'dISt'
	JSR LODMSG

PINPUT	JSR KDOWN	; DISPLAY MSG AND CHECK FOR KEY PRESS
	JSR $1F6A	; GET A KEY
	CMP #$0A	
	BCS PINPUT	; IF >=#10, TRY AGN
	STA $83		; STASH THE VALUE OF THE KEY PRESS
	ASL		; MULTIPLY AC (KEY) BY 8
	ASL
	ASL
	STA $84		; STORE IN TEMP AREA
	LDA $83
	ASL		; 2X AND STORE IN TEMP AREA
	CLC
	ADC $84		; ADD AC ($83x2) AND $84 FOR 10s DIGIT
	STA STOREA	; STORE IN TEMP FOR AC
	
PINPT2	JSR KDOWN	; DISPLAY MSG AND CHECK FOR KEY PRESS
	JSR $1F6A	; GET A KEY
	CMP #$0A	
	BCS PINPT2	; IF >=#10, TRY AGN
	CLC
	ADC STOREA	; ADD THE 1s TO THE 10s IN AC TEMP 
	STA TARGET	; AND RE-STORE IT
	RTS

; TEST SECTION

TESTIT	LDA TARGET
	CMP #$04	; target inside player fort?
	BCS TEST2	; no, next test
	JMP SUICID	; yes, blew self up! Loser!!!!
TEST2	CMP #$32	; threw past limit? #50 (meters)
	BCC GUDLOB	; no, OK
	LDA #$32	; yes, put the throw at the limit
	STA TARGET

GUDLOB	JSR LOBBIT	; LAUNCH the grenade!
	LDA GRNADE
	SEC
	SBC #$01	; one less grenade
	STA GRNADE

GOTHIM	SEC		; DID PLAYER GET HIM?
	LDA TARGET
	SBC NMEPOS
	BNE PLUS1	; if <>00 then next test
	JMP WINNER	; it's a match - winner!
PLUS1	CMP #$01	; is enemy +1 away?
	BNE MINUS1	; no, next test
	JMP WINNER	; close enough!
MINUS1	CMP #$FF	; is enemy -1 away?
	BNE TEST3	; no, then goto short/long test
	JMP WINNER	; just like horseshoes and nuclear war!

TEST3	LDA GRNADE	; see if out of grenades
	BNE DELTA	; if not, move NME
	JMP LOSER	; but if no grenades.....

DELTA	SEC
	LDA NMEPOS
	SBC TARGET
	CMP #$40
	BCS LNGLOB	; if result of subtraction is over $B0, SHORT
SHTLOB	JSR MOVNME	; Move enemy
	LDX #$05	; SET UP LOOP COUNTER
	STX STOREX
LOOP1	LDY #$06
	JSR LODMSG	; set up MSG#2 'Short' and display
	JSR FIXTAR
	JSR DELAY1
	LDX STOREX
	DEX
	STX STOREX
	BNE LOOP1
	JSR NUMLFT
	RTS		; RTS to MAIN loop

LNGLOB	JSR MOVNME	; Move enemy	
	LDX #$05	; SET UP LOOP COUNTER
	STX STOREX
LOOP2	LDY #$0C
	JSR LODMSG	; set up MSG#3 - 'Long' and display
	JSR FIXTAR
	JSR DELAY1
	LDX STOREX
	DEX
	STX STOREX
	BNE LOOP2
	JSR NUMLFT
	RTS		; RTS to MAIN loop

MOVNME	JSR RAND_8	; get a random number 00-FF in A
	AND #$07	; mask to keep bottom 3 bits (<#08) 
	BEQ MOVNME	; if 00, get another rnd #
	STA STOREA	; put the move distance in temp loc.
	LDA NMEPOS	; load orig enemy position
	SEC		
	SBC STOREA	; subtract move dist from orig. pos.
	STA NMEPOS	; store new pos. in 'Enemy Pos.' location
	CMP #$09
	BCS DONE5	; is enemy w/i 5 yrds of the player's bunker?
	JMP LOSER	; yes, ...
DONE5	RTS		; otherwise, RTS to finish the main loop

FIXTAR	JSR HX2DEC	; CONVERT TARGET THROW TO DEC
	STA STOREA	; STORE RESULT
	LSR		; MSB
	LSR
	LSR
	LSR
	TAX
	LDA NUMBER,X
	STA $90		; STORE IT 5TH DISPLAY POS.
	LDA STOREA	; GET BYTE BACK
	AND #$0F	; MASK FOR LSB
	TAX
	LDA NUMBER,X
	STA $91		; STORE IT 6TH DISPLAY POS.
	RTS

NUMLFT	LDY #$48	; MSG #13
	JSR LODMSG
	LDX GRNADE	; GET # OF GRENADES LEFT
	LDA NUMBER,X
	STA $91		; STORE IN 6TH DISPLAY POS.
	JSR DELAY1
	JSR DELAY1
	JSR DELAY1
	JSR DELAY1
	RTS

; NEEDS Y SENT TO ROUTINE TO SET APPROPRIATE OUTPUT MSG

LODMSG	LDX #$00
LOADIT	LDA TABLE,Y
	STA $8C,X
	INY
	INX
	CPX #$06
	BNE LOADIT
	RTS

DELAY1	LDA #$00	; 1 CASCADE CYCLE TO $82 = +/- .7 SEC
	STA $80
	STA $81
	CLC
FIRST	LDA $80
	ADC #$01
	STA $80
	JSR DISP
	BNE FIRST
SECOND	LDA $81
	ADC #$00
	STA $81
	CMP #$6B	; ABOUT .4 SEC
	BCC FIRST
	RTS

; DISP, KDOWN, and CLRDIS modified from Dan Lewart's 
; "Card Dealer" in The First Book of KIM, page 48.

CLRDIS	LDX     #$06            ; CLEAR DISPLAY MATRIX
        LDY     #$00            ; (8C-91)=0
CLEAR1	STY     $008B,X
        DEX
        BNE     CLEAR1
	RTS

DISP    LDA     #$7F            ; SET SEGMENTS TO OUTPUT
        STA     $1741
        LDY     #$00            ; INITIALIZE
        LDX     #$08
DISP1   LDA     $008C,Y         ; GET CHARACTER
        STY     $00FC
        JSR     $1F4E           ; DISPLAY CHARACTER
        INY                     ; NEXT CHARACTER
        CPY     #$06 
        BCC     DISP1
	RTS

KDOWN   JSR     DISP2           ; DISPLAY (8C-91)
        BNE     KDOWN           ; UNTIL KEY UP
KUP     JSR     DISP2           ; DISPLAY (8C-91)
        BNE     GOBACK          ; UNTIL KEY DOWN
        BEQ     KUP
DISP2   LDA     #$7F            ; SEGMENTS TO OUTPUT
        STA     $1741
        LDY     #$00            ; INITIALIZE
        LDX     #$08
DISP3   LDA     $008C,Y         ; GET CHARACTER
        STY     $00FC
        JSR     $1F4E           ; DISPLAY CHARACTER
        INY                     ; NEXT CHARACTER
        CPY     #$06 
        BCC     DISP3
        JMP     $1F3D           ; DONE, KEY DOWN?
GOBACK	RTS

; SIMPLE HEX -> DEC CONVERSION BY JACK CLARKE
; From "COMPUTE!" magazine, Issue 10, Pg. 60, March 1981
;
; Only Hex $00-$63 and Decimal #00-#99 work with this method. [?]

HX2DEC	SED		; SET DEC MODE
	CLC		; CLEAR CARRY
	LDA #$00	; CLEAR AC
	LDX TARGET	; HEX NUM TO CNVRT
ADD1	ADC #$01	; ADD 1 TO AC
	DEX		; SUBTRACT 1 FROM X
	BNE ADD1	; IS X ZERO?, NO, BRANCH TO ADD1
	CLD		; RESULT IS IN AC
	RTS

	; SIMPLE 8-BIT PSEUDO-RANDOM NUMBER GENERATOR
	; This next part (for any 6502) by Lee Davison, 2002.
	; from https://philpem.me.uk/leeedavison/6502/prng/index.html

rand_8
	LDA	r_seed		; get seed [THIS PROG. FROM 1 uSEC TIMER]
	AND	#$B8		; mask non feedback bits
				; for max length run with 8 bits need
				; taps at b7, b5, b4 and b3
	LDX	#$05		; bit count (shift top 5 bits)
	LDY	#$00		; clear feedback count
F_loop
	ASL	A		; shift bit into carry
	BCC	bitclr		; branch if bit = 0

	INY			; inc feedback count (b0 is XOR all the
				; shifted bits from A)
bitclr
	DEX			; decrement count
	BNE	F_loop		; loop if not all done

no_clr
	TYA			; copy feedback count
	LSR	A		; bit 0 into Cb
	LDA	r_seed		; get seed back
	ROL	A		; rotate carry into byte
	STA	r_seed		; save number as next seed
	RTS			; done
	
r_seed				; prng seed byte (must not be zero)
.byte	$01, $20, $62, $79, $20, $4B, $37, $43, $43
.byte	$43, $20, $31, $2F, $27, $32, $33, $20

; ====================================================

* = $0100

WINNER	CLD
	LDY #$18	; SET UP FOR MSG 5 - '--86Ed'
	STY STOREY
	JSR MESAGE
	JMP BYEBYE
	
LOSER	CLD
	LDY #$12	; SET UP FOR MSG 4 - '9ot4ou'
	STY STOREY
	JSR MESAGE
	JMP BYEBYE

BYEBYE	CLD
	LDY #$54	; MSG #14
	JSR LODMSG
	JSR DELAY1
	JSR DELAY1
	JSR DELAY1
	JSR DELAY1
	BRK

MESAGE	LDA #$00
	STA $83		; LOOP COUNTER
BLINKY	LDY STOREY	; LOAD MSG #6
	JSR LODMSG
	JSR DELAY1	; INCLUDES OUTPUT TO DISPLAY
	JSR DELAY1
	JSR CLRDIS	; BLANK THE DISPLAY
	JSR DELAY1
	JSR DELAY1
	CLC
	LDA $83
	ADC #$01
	STA $83
	CMP #$03	; LOOP 3X
	BNE BLINKY
	RTS

TABLE
.BYTE	$5E, $06, $6D, $78, $08, $08	; 'dISt__' $00
.BYTE	$6D, $74, $50, $78, $00, $00	; '5hrt  ' $06
.BYTE	$38, $5C, $54, $6F, $00, $00	; 'Lon9  ' $0C
.BYTE	$6F, $5C, $78, $66, $5C, $1C	; '9ot4ou' $12
.BYTE	$40, $40, $7F, $7D, $79, $5E	; '--86Ed' $18

.BYTE	$38, $5C, $7C, $7C, $79, $50	; 'LobbEr' $1E

.BYTE	$08, $00, $00, $00, $00, $00	; '_     ' $24
.BYTE	$00, $40, $00, $00, $00, $00	; ' -    ' $2A
.BYTE	$00, $00, $01, $00, $00, $00	; '  ^   ' $30
.BYTE	$00, $00, $00, $01, $00, $00	; '   ^  ' $36
.BYTE	$00, $00, $00, $00, $40, $00	; '    - ' $3C
.BYTE	$00, $00, $00, $00, $00, $08	; '     _' $42

.BYTE	$38, $79, $71, $78, $00, $00	; 'LEFt  ' $48
.BYTE	$3E, $50, $71, $5C, $50, $78	; 'UrFort' $4E
.BYTE	$38, $5F, $78, $79, $50, $08	; 'LAtEr_' $54

NUMBER
.BYTE   $3F, $06, $5B, $4F, $66, $6D, $7D, $07, $7F, $6F

; =============================================

* = $1780

SUICID	CLD
	LDA #$24	; START FOR MSGS 7-9 (RISING GRENADE)
	STA $83
LOB2	TAY
	JSR LODMSG
	JSR DELAY1
	CLC
	LDA $83		; GET BACK THE MSG # AND +6
	ADC #$06	; (ALL MSGS ARE 6 CHAR.)
	STA $83
	CMP #$36	; DON'T WANT MSG #10
	BNE LOB2
LESLOB	SEC
	LDA $83
	SBC #$06	; NOW MSGS 9-7 (GRENADE FALLS *BACK*)
	STA $83
	CMP #$1E	; DID WE GO PAST THE GRENADE GRAPHICS?
	BEQ URFORT	; YES, GOTO PRINT "You IDIOT!!"
	TAY
	JSR LODMSG
	JSR DELAY1
	JMP LESLOB
URFORT	LDY #$4E	; SET UP FOR MSG 14 - 'UrFort'
	STY STOREY
	JSR MESAGE
	JMP BYEBYE