;
; GRENADE LOBBER - by Dave Hassler, Nov. 2022
;
; an "animated" game for PAL-1/microKIM.
;
; for best results, use 80x24 terminal screen, serial emulator at 4800 bps,
; and terminal emulator cursor either blank or an underscore.
;
; Released "to the wild" under the MIT License and copyright 2022 by
; David Hassler.  Please feel free to augment, fix, change, and make a 
; derivative work - just give credit for the original version.
;
; As Liu Ganning says: "Happy Hacking!"  --  DHH
;


FIELD  = $1780
STOREA = $20
NMEPOS = $21
MISSBY = $22
GRNADE = $23
TARGET = $24
STOREX = $25
GOTHIT = $26


*=$0200

; CLEAR "SCREEN"
	CLD		; BECAUSE IT'S NICE
	LDA #$FF
	STA $F2		; RESET STACK POINTER
	LDX #$19	; 25 LINES-WORTH OF CRLF
	JSR CLEAR

; PRINT TITLE/INSTR.

	LDX #$00
PRNT1	LDA TITLE,X
	BEQ SEEDST	; out to get an initial RND seed
	JSR $1EA0
	INX
	JMP PRNT1

SEEDST	LDA $1744	; from 1 u-sec timer in the RIOT
	BEQ SEEDST	; if #00, get another 'seed'
	STA R_SEED	; store it in main RND routine seed holder


	LDX #$0A	; 10 more CRLFs to set up for play
	JSR CLEAR

; PRINT 'HEADER' OF 'YARD MARKERS'
	
	CLC
	LDA #$00
PRTYRD	JSR $1E3B	; print the byte
	ADC #$10	; add 16 (but = $20, so...  :^)
	CMP #$60	; is it $60?
	BEQ HDR2	; yes, branch to print rest of header
	STA $20		; otherwise, put A somewhere safe
	LDX #$08
SPCOUT	JSR $1E9E	; call OUTSP
	DEX
	BNE SPCOUT	; not #00, loop
	LDA $20		; otherwise, reload A
	BNE PRTYRD	; will always branch to print new byte

; MORE HEADER TEXT - STATUS BAR
HDR2	LDX #$00
PRNT2	LDA HEDTXT,X
	BEQ ONECR	; out to print CRLF and finsh header
	JSR $1EA0
	INX
	JMP PRNT2

ONECR	LDX #$01
	JSR CLEAR


; POSITION THE ENEMY BETWEEN #29-59, SET INIT # GRENADES, DIST TO ENEMY

NME1ST	JSR RAND_8	; get a random number 00-FF in A
	AND #$3F	; mask to keep bottom 6 bits (<#64) 
	CMP #$14	; is A <#20?
	BCC NME1ST	; yes, get another rnd #
	CMP #$32	; is A >#50?
	BCS NME1ST	; yes, get another rnd #
	STA NMEPOS	; store the AC in 'Enemy Pos.' location
;	LDA #$30	; TEMP/TEST: PUT HIM AT #48
;	STA NMEPOS
	LDA #$03	; HEX #
	STA GRNADE	; number of grenades

; SET UP INITAL PLAYING FIELD, VARIABLES

	CLD
HDR3	LDX #$00
POKE3	LDA URBASE,X
	BEQ FILFLD	; done 'poking' in player base
	STA FIELD,X
	INX
	JMP POKE3

FILFLD	LDX #$05	; LOAD UP REST OF FIELD WITH 'GROUND'
FLD2	LDA #$5F	; load up a '_'  to make 'ground'
	STA FIELD,X
	INX
	CPX #$39	; is it #57? (end of field of play +4)
	BNE FLD2
	LDA #$7C
	STA $17B4	; put the pipe char into the field
	LDA #$00	
	STA FIELD,X	; TERMINATE WITH ZERO AT END OF LINE
			; fall through to main game loop






; *** M A I N   L O O P ***

GMLOOP	JSR PRTFLD	; print the current field
	JSR GETTRG	; get new target for lob, test, move enemy
	JMP GMLOOP
	BRK





; *** SUBROUTINES

; LOB GRENADE

LOBDOT	LDX #$05	; LOOP COUNTER
LOBSTR	LDA #$2E	; LOAD A DOT
	STA FIELD,X
	STX STOREX	; store, because of OUTCH loop
	JSR PRTFLD	; go print the field
	LDX STOREX	; restore X
	LDA #$5F	; load 'ground'
	STA FIELD,X	; restore at curr. loc
	LDA FIELD+1,X	; get what's in NEXT field loc
	CMP #$2A	; is it '*'?
	BNE BUMPX	; no, inx and see if target reached
	
	INX		; test to see if curr target is a prev trgt
	CPX TARGET
	BNE BUMPX
	DEX
	JMP GOFELD

EXTRAX	INX		; extra INX to skip over the '*'
BUMPX	INX
	CPX #$40	; test for runaway condx when TRGT is <#10
	BEQ GOFELD
	CPX TARGET	
	BNE LOBSTR	; fill with '~' until -1 target
	LDA #$2A
	STA FIELD,X	; store a '*' at target
GOFELD	JSR PRTFLD
	RTS


; PRINT THE FIELD

PRTFLD	LDA #$0D	; load a CR
	JSR $1EA0	; just in case....
	LDX #$00
LOOP5	LDA FIELD,X	; get data in a field loc.
	JSR $1EA0	; print it
	INX		; + 1
	CPX #$39	; is X #57?
	BNE LOOP5	; no, loop
	JSR SPACE3
	LDA GRNADE
	JSR $1E3B	; print # of grenades left
	JSR SPACE3	; prepare for player input
	RTS


; GET PLAYER INPUT AND TEST

GETTRG	JSR $1F9D	; call GETBYT
	STA TARGET
	LDA #$0D	; print a CR
	JSR $1EA0

; TEST - LOB GRENADE AND CHECK FOR RESULT

GUDTRG	JSR DEC2HX	; convert the input 'byte' to hex
	STX TARGET	; place the result in target
	LDA TARGET
	CMP #$04	; target inside player fort?
	BCS TEST2	; no, next test
	JMP SUICID	; yes, blew self up!
TEST2	CMP #$32	; threw past limit?
	BCC GUDLOB	; no, OK
	LDA #$32	; yes, put the throw at the limit
	STA TARGET

GUDLOB	JSR LOBDOT	; LAUNCH the grenade!
	LDA GRNADE
	SEC
	SBC #$01	; one less grenade
	STA GRNADE


GOTHIM	SEC		; DID PLAYER GET HIM?
	LDA TARGET
	SBC NMEPOS
	BNE PLUS1	; if <>00 then next test
	JMP WINNER	; it's a match - winner!
PLUS1	CMP #$01	; is enemy +1 away?
	BNE MINUS1	; no, next test
	JMP WINNER	; close enough!
MINUS1	CMP #$FF	; is enemy -1 away?
	BNE DELTA	; no, then goto short/long test
	JMP WINNER	; just like horseshoes and nuclear war!

DELTA	CLC
	CMP #$B0	; DETERMINE SHORT OR LONG LOB
	BCC BIGLOB	; if result of subtraction is over $80, 
	LDX #$00	; then the throw was SHORT (?)
	BEQ AD2FLD	; set offset X to start of MISTXT and branch
BIGLOB	LDX #$05
AD2FLD	LDY #$35	; *** WAS $3F WHEN 60-YD FIELD
LOOP7	LDA MISTXT,X	; get a char
	BEQ TEST3	; #00?  out
	STA FIELD,Y	; poke it into the field
	INX
	INY
	JMP LOOP7	; 


TEST3	LDA GRNADE	; see if out of grenades
	BNE MOVNME	; if not, move NME
	JMP LOSER	; but if no grenades.....


; MOVE THE ENEMY 1-6 YRDS, SEE IF < 5 YRDS AWAY, AND COMPLETE THE TURN

MOVNME	JSR RAND_8	; get a random number 00-FF in A
	AND #$07	; mask to keep bottom 3 bits (<#08) 
	BEQ MOVNME	; if 00, get another rnd #
	STA STOREA	; put the move distance in temp var
	LDA NMEPOS	; load orig enemy position
	SEC		
	SBC STOREA	; subtract move dist from orig. pos.
	STA NMEPOS	; store new pos. in 'Enemy Pos.' location
	CMP #$09
	BCS DONE5	; is enemy w/i 5 yrds of the player's bunker?
	JMP LOSER	; yes, ...
DONE5	RTS		; otherwise, RTS to finish the main loop



CLEAR	LDA #$0A
	JSR $1EA0
	LDA #$0D
	JSR $1EA0
	DEX
	BNE CLEAR
	RTS

SPACE3	LDX #$03
LOOP6	JSR $1E9E	; CALL OUTSP
	DEX
	BNE LOOP6
	RTS


WINNER	CLD
	LDX #$00
	LDY #$0E	
POKE4	LDA WINTXT,X	
	BEQ DONE2	; done 'poking' in message
	STA FIELD,Y	; start at #14 on field
	INX
	INY
	JMP POKE4
DONE2	JSR PRTFLD	; print the ending message on the field
	JSR $1E2F	; call CRLF
	BRK
	

LOSER	CLD
	LDX #$00	
	LDY #$0E
POKE5	LDA LTEXT,X	
	BEQ DONE3	; done 'poking' in message
	STA FIELD,Y	; start at #19 on field
	INX
	INY
	JMP POKE5
DONE3	JSR PRTFLD	; print the ending message on the field
	JSR $1E2F	; call CRLF
	BRK


SUICID	CLD
	LDX #$00
	LDY #$0E	
POKE6	LDA DEDTXT,X	
	BEQ DONE4	; done 'poking' in message
	STA FIELD,Y	; start at #19 on field
	INX
	INY
	JMP POKE6
DONE4	JSR PRTFLD	; print the ending message on the field
	JSR $1E2F	; call CRLF
	BRK


; SIMPLE HEX <-> DEC CONVERSION BY JACK CLARKE
; From "COMPUTE!" magazine, Issue 10, Pg. 60, March 1981
;
; Only Hex $00-$63 and Decimal #00-#99 work with this method. [?]

DEC2HX	SED		; SET DEC MODE
	SEC		; CLEAR CARRY
	LDX #$00	; CLEAR X
	LDA TARGET	; 'DECIMAL' NUMBER TO CNVRT
SUB1	SBC #$01	; SUBTRACT 1 FROM AC
	INX		; ADD 1 TO X
	CMP #$00	; IS AC ZERO?
	BNE SUB1	; NO, BRANCH TO SUB1
			; was INX (NEEDS A +1 BUMP) [but doesn't]
	CLD		; RESULT IS IN X
	RTS

HX2DEC	SED		; SET DEC MODE
	CLC		; CLEAR CARRY
	LDA #$00	; CLEAR AC
	LDX MISSBY	; HEX NUM TO CNVRT
ADD1	ADC #$01	; ADD 1 TO AC
	DEX		; SUBTRACT 1 FROM X
	BNE ADD1	; IS X ZERO?, NO, BRANCH TO ADD1
	CLD		; RESULT IS IN AC
	RTS





	; SIMPLE 8-BIT PSEUDO-RANDOM NUMBER GENERATOR
	; This next part (for any 6502) by Lee Davison, 2002.
	; from https://philpem.me.uk/leeedavison/6502/prng/index.html

rand_8
	LDA	r_seed		; get seed [THIS PROG. FROM 1 uSEC TIMER]
	AND	#$B8		; mask non feedback bits
				; for maximal length run with 8 bits we need
				; taps at b7, b5, b4 and b3
	LDX	#$05		; bit count (shift top 5 bits)
	LDY	#$00		; clear feedback count
F_loop
	ASL	A		; shift bit into carry
	BCC	bitclr		; branch if bit = 0

	INY			; increment feedback count (b0 is XOR all the
				; shifted bits from A)
bitclr
	DEX			; decrement count
	BNE	F_loop		; loop if not all done

no_clr
	TYA			; copy feedback count
	LSR	A		; bit 0 into Cb
	LDA	r_seed		; get seed back
	ROL	A		; rotate carry into byte
	STA	r_seed		; save number as next seed
	RTS			; done
	
r_seed
	.byte	1		; prng seed byte (must not be zero)



TITLE
.BYTE "GRENADE LOBBER - by Dave Hassler (80-col, 4800 bps)"
.BYTE $0A $0D
.BYTE "You have only 3 grenades. The hidden enemy advances 1-7 m/turn"
.BYTE $0A $0D
.BYTE "and may start near or down range. Enter your launch distance"
.BYTE $0A $0D
.BYTE "at 'Trgt'. +/-1 m is a hit. Status report each turn. Good luck!"
.BYTE $00

HEDTXT
.BYTE "|Miss Left Trgt"
.BYTE $00

URBASE
.BYTE "[_/_]"
.BYTE $00

WINTXT
.BYTE " TARGET ELIMINATED - VICTORY! "
.BYTE $00

LTEXT
.BYTE " MISS - NO GRENADES - DEFEAT! "
.BYTE $00

DEDTXT
.BYTE " YOU BLEW YOURSELF UP, IDIOT! "
.BYTE $00

MISTXT
.BYTE "SHRT"
.BYTE $00
.BYTE "LONG"
.BYTE $00

